 /* AUTORIGHTS
 Copyright (c) 2008 The Regents of the University of California.
 All Rights Reserved.
 
 Created by Stefano Carpin
 University of California, Merced, Robotics Lab - School of Engineering
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for educational, research and non-profit purposes,
 without fee, and without a written agreement is hereby granted,
 provided that the above copyright notice, this paragraph and the
 following three paragraphs appear in all copies.
 
 This software program and documentation are copyrighted by The Regents
 of the University of California. The software program and
 documentation are supplied "as is", without any accompanying services
 from The Regents. The Regents does not warrant that the operation of
 the program will be uninterrupted or error-free. The end-user
 understands that the program was developed for research purposes and
 is advised not to rely exclusively on the program for any reason.
 
 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF
 CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE
 MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

*/


#include "scan3d.h"
#include "hough3d.h"
#include "io.h"

#include <iostream>
#include <vector>
#include <sys/time.h>
#include <time.h>


using namespace std;
using namespace h3d;


int main(int argc,char **argv) {

  if ( argc != 3 ) {
    cout << "Please provide the name of two files in json format with the scan data" << endl;
    return 1;
  }
  
  scan3D s1;
  s1.load_scan_jsonfile(argv[1]);
  
  scan3D s2;
  s2.load_scan_jsonfile(argv[2]);
  

  h3d_params p;

  unsigned int i,j;
  double max1 = -1,max2 = -1;
    
  for ( i = 0 ; i < s1.get_nscans() ; i++ ) {
    for ( j = 0 ; j < s1.get_nbeams() ; j++ ) {
      if ( s1.valid[i][j] )
	max1 = s1.readings[i][j] > max1 ? s1.readings[i][j] : max1;
      if ( s2.valid[i][j] )
	max2 = s2.readings[i][j] > max2 ? s2.readings[i][j] : max2;
    }
  }
  
  p.max_norm = 1.1 * ( max1 > max2 ? max1 : max2 );

  
  matrix33 identity;
  point zero_point(0,0,0);
  
  h3d_struct ht1(p.angular_cell_size_deg,p.rho_cell_size,-p.max_norm,p.max_norm);
  h3d_struct ht2(p.angular_cell_size_deg,p.rho_cell_size,-p.max_norm,p.max_norm);
  
  ht1.compute_cube_ht_normals(s1,identity,zero_point);
  ht2.compute_cube_ht_normals(s2,identity,zero_point);
  
  list<rotation_hypothesis> hyp = ht1.guess_rotation(ht2);

  deque<translation_hypothesis> tran;
  list<roto_translation_hypothesis> roto_tran;
  roto_translation_hypothesis tmp_rototran;
  
  list<rotation_hypothesis>::iterator r_it;
  for ( r_it = hyp.begin() ; r_it != hyp.end() ; r_it++ ) {
    tran = ht1.guess_translation(ht2,r_it->rotation);
    tmp_rototran.rot = *r_it;
    for ( unsigned int i = 0 ; i < tran.size() ; i++ ) {
      tmp_rototran.tran = tran[i];
      roto_tran.push_back(tmp_rototran);
    }
  }

  vector<ranked_solution> v_final = ht1.evaluate_solutions_hausdorff(s1,s2,identity,zero_point,roto_tran);

  cout << v_final[0] << " " << v_final[1] << " " << v_final[2] << endl;
  
  return 0;

}
